@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2020 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Client.hxx"
 #include "Config.hxx"
@@ -26,13 +10,13 @@
 #include <cstring>
 
 BufferedSocket::InputResult
-Client::OnSocketInput(void *data, size_t length) noexcept
+Client::OnSocketInput(std::span<std::byte> src) noexcept
 {
 	if (background_command)
 		return InputResult::PAUSE;
 
-	char *p = (char *)data;
-	char *newline = (char *)std::memchr(p, '\n', length);
+	char *p = (char *)src.data();
+	char *newline = (char *)std::memchr(p, '\n', src.size());
 	if (newline == nullptr)
 		return InputResult::MORE;
 
