@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "CommandError.hxx"
@@ -27,8 +11,7 @@
 
 #include <system_error>
 
-gcc_const
-static enum ack
+static constexpr enum ack
 ToAck(PlaylistResult result) noexcept
 {
 	switch (result) {
@@ -63,8 +46,8 @@ ToAck(PlaylistResult result) noexcept
 }
 
 #ifdef ENABLE_DATABASE
-gcc_const
-static enum ack
+
+static constexpr enum ack
 ToAck(DatabaseErrorCode code) noexcept
 {
 	switch (code) {
@@ -78,9 +61,10 @@ ToAck(DatabaseErrorCode code) noexcept
 
 	return ACK_ERROR_UNKNOWN;
 }
+
 #endif
 
-gcc_pure
+[[gnu::pure]]
 static enum ack
 ToAck(const std::exception_ptr& ep) noexcept
 {
