@@ -1,28 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "MessageCommands.hxx"
 #include "Request.hxx"
 #include "client/Client.hxx"
 #include "client/List.hxx"
 #include "client/Response.hxx"
-#include "util/ConstBuffer.hxx"
 #include "Partition.hxx"
 
 #include <fmt/format.h>
@@ -34,7 +17,7 @@
 CommandResult
 handle_subscribe(Client &client, Request args, Response &r)
 {
-	assert(args.size == 1);
+	assert(args.size() == 1);
 	const char *const channel_name = args[0];
 
 	switch (client.Subscribe(channel_name)) {
@@ -62,7 +45,7 @@ handle_subscribe(Client &client, Request args, Response &r)
 CommandResult
 handle_unsubscribe(Client &client, Request args, Response &r)
 {
-	assert(args.size == 1);
+	assert(args.size() == 1);
 	const char *const channel_name = args[0];
 
 	if (client.Unsubscribe(channel_name))
@@ -78,7 +61,7 @@ handle_channels(Client &client, [[maybe_unused]] Request args, Response &r)
 {
 	assert(args.empty());
 
-	std::set<std::string> channels;
+	std::set<std::string, std::less<>> channels;
 
 	for (const auto &c : client.GetPartition().clients) {
 		const auto &subscriptions = c.GetSubscriptions();
@@ -109,7 +92,7 @@ handle_read_messages(Client &client,
 CommandResult
 handle_send_message(Client &client, Request args, Response &r)
 {
-	assert(args.size == 2);
+	assert(args.size() == 2);
 
 	const char *const channel_name = args[0];
 	const char *const message_text = args[1];
