@@ -1,31 +1,18 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Configured.hxx"
 #include "DatabaseGlue.hxx"
 #include "Interface.hxx"
 #include "config/Data.hxx"
+#include "config/Domain.hxx"
 #include "config/Param.hxx"
 #include "config/Block.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "fs/StandardDirectory.hxx"
-#include "util/RuntimeError.hxx"
+#include "fs/FileSystem.hxx"
+#include "fs/glue/StandardDirectory.hxx"
+#include "lib/fmt/RuntimeError.hxx"
+#include "Log.hxx"
 
 DatabasePtr
 CreateConfiguredDatabase(const ConfigData &config,
@@ -36,8 +23,8 @@ CreateConfiguredDatabase(const ConfigData &config,
 	const auto *path = config.GetParam(ConfigOption::DB_FILE);
 
 	if (param != nullptr && path != nullptr)
-		throw FormatRuntimeError("Found both 'database' (line %d) and 'db_file' (line %d) setting",
-					 param->line, path->line);
+		throw FmtRuntimeError("Found both 'database' (line {}) and 'db_file' (line }) setting",
+				      param->line, path->line);
 
 	if (param != nullptr) {
 		param->SetUsed();
@@ -51,17 +38,32 @@ CreateConfiguredDatabase(const ConfigData &config,
 	} else {
 		/* if there is no override, use the cache directory */
 
-		const AllocatedPath cache_dir = GetUserCacheDir();
-		if (cache_dir.IsNull())
+		const AllocatedPath cache_dir = GetAppCacheDir();
+		if (cache_dir.IsNull()) {
+			FmtDebug(config_domain, "No cache directory; disabling the database");
 			return nullptr;
+		}
 
-		const auto db_file = cache_dir / Path::FromFS(PATH_LITERAL("mpd.db"));
+		const auto db_file = cache_dir / Path::FromFS(PATH_LITERAL("db"));
 		auto db_file_utf8 = db_file.ToUTF8();
 		if (db_file_utf8.empty())
 			return nullptr;
 
 		ConfigBlock block;
 		block.AddBlockParam("path", std::move(db_file_utf8), -1);
+
+		{
+			const auto mounts_dir = cache_dir
+				/ Path::FromFS(PATH_LITERAL("mounts"));
+			CreateDirectoryNoThrow(mounts_dir);
+
+			if (auto mounts_dir_utf8 = mounts_dir.ToUTF8();
+			    !mounts_dir_utf8.empty())
+				block.AddBlockParam("cache_directory",
+						    std::move(mounts_dir_utf8),
+						    -1);
+		}
+
 		return DatabaseGlobalInit(main_event_loop, io_event_loop,
 					  listener, block);
 	}
