@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "DatabasePrint.hxx"
 #include "Selection.hxx"
@@ -24,6 +8,7 @@
 #include "client/Response.hxx"
 #include "Partition.hxx"
 #include "song/LightSong.hxx"
+#include "tag/Names.hxx"
 #include "tag/Tag.hxx"
 #include "LightDirectory.hxx"
 #include "PlaylistInfo.hxx"
@@ -36,7 +21,7 @@
 
 #include <functional>
 
-gcc_pure
+[[gnu::pure]]
 static const char *
 ApplyBaseFlag(const char *uri, bool base) noexcept
 {
@@ -191,11 +176,11 @@ PrintSongUris(Response &r, Partition &partition,
 }
 
 static void
-PrintUniqueTags(Response &r, ConstBuffer<TagType> tag_types,
+PrintUniqueTags(Response &r, std::span<const TagType> tag_types,
 		const RecursiveMap<std::string> &map) noexcept
 {
 	const char *const name = tag_item_names[tag_types.front()];
-	tag_types.pop_front();
+	tag_types = tag_types.subspan(1);
 
 	for (const auto &[key, tag] : map) {
 		r.Fmt("{}: {}\n", name, key);
@@ -207,7 +192,7 @@ PrintUniqueTags(Response &r, ConstBuffer<TagType> tag_types,
 
 void
 PrintUniqueTags(Response &r, Partition &partition,
-		ConstBuffer<TagType> tag_types,
+		std::span<const TagType> tag_types,
 		const SongFilter *filter)
 {
 	const Database &db = partition.GetDatabaseOrThrow();
