@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "Control.hxx"
@@ -29,11 +13,11 @@
 #include "input/InputStream.hxx"
 #include "input/Registry.hxx"
 #include "DecoderList.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "system/Error.hxx"
 #include "util/MimeType.hxx"
 #include "util/UriExtract.hxx"
 #include "util/UriUtil.hxx"
-#include "util/RuntimeError.hxx"
 #include "util/Domain.hxx"
 #include "util/ScopeExit.hxx"
 #include "util/StringCompare.hxx"
@@ -47,12 +31,46 @@
 
 static constexpr Domain decoder_thread_domain("decoder_thread");
 
+enum class DecodeResult {
+	/**
+	 * No plugin supporting this file type was found.
+	 */
+	NO_PLUGIN,
+
+	/**
+	 * A plugin was found, but it does not support streaming.
+	 */
+	NO_STREAM_PLUGIN,
+
+	/**
+	 * A plugin was found, but it did not recgonize the file.
+	 */
+	UNRECOGNIZED_FILE,
+
+	/**
+	 * A "stop" command was found before decoder initialization
+	 * was completed.
+	 */
+	STOP,
+
+	/**
+	 * The file was decoded successfully.
+	 */
+	SUCCESS,
+};
+
+static constexpr bool
+IsFinalDecodeResult(DecodeResult result) noexcept
+{
+	return result >= DecodeResult::STOP;
+}
+
 /**
  * Decode a URI with the given decoder plugin.
  *
  * Caller holds DecoderControl::mutex.
  */
-static bool
+static DecodeResult
 DecoderUriDecode(const DecoderPlugin &plugin,
 		 DecoderBridge &bridge, const char *uri)
 {
@@ -65,12 +83,12 @@ DecoderUriDecode(const DecoderPlugin &plugin,
 	FmtDebug(decoder_thread_domain, "probing plugin {}", plugin.name);
 
 	if (bridge.dc.command == DecoderCommand::STOP)
-		throw StopDecoder();
+		return DecodeResult::STOP;
 
 	{
 		const ScopeUnlock unlock(bridge.dc.mutex);
 
-		FormatThreadName("decoder:%s", plugin.name);
+		FmtThreadName("decoder:{}", plugin.name);
 
 		plugin.UriDecode(bridge, uri);
 
@@ -80,7 +98,9 @@ DecoderUriDecode(const DecoderPlugin &plugin,
 	assert(bridge.dc.state == DecoderState::START ||
 	       bridge.dc.state == DecoderState::DECODE);
 
-	return bridge.dc.state != DecoderState::START;
+	return bridge.dc.state == DecoderState::START
+	       ? DecodeResult::UNRECOGNIZED_FILE
+	       : DecodeResult::SUCCESS;
 }
 
 /**
@@ -88,7 +108,7 @@ DecoderUriDecode(const DecoderPlugin &plugin,
  *
  * Caller holds DecoderControl::mutex.
  */
-static bool
+static DecodeResult
 decoder_stream_decode(const DecoderPlugin &plugin,
 		      DecoderBridge &bridge,
 		      InputStream &input_stream,
@@ -103,7 +123,7 @@ decoder_stream_decode(const DecoderPlugin &plugin,
 	FmtDebug(decoder_thread_domain, "probing plugin {}", plugin.name);
 
 	if (bridge.dc.command == DecoderCommand::STOP)
-		throw StopDecoder();
+		return DecodeResult::STOP;
 
 	/* rewind the stream, so each plugin gets a fresh start */
 	try {
@@ -114,7 +134,7 @@ decoder_stream_decode(const DecoderPlugin &plugin,
 	{
 		const ScopeUnlock unlock(bridge.dc.mutex);
 
-		FormatThreadName("decoder:%s", plugin.name);
+		FmtThreadName("decoder:{}", plugin.name);
 
 		plugin.StreamDecode(bridge, input_stream);
 
@@ -124,7 +144,9 @@ decoder_stream_decode(const DecoderPlugin &plugin,
 	assert(bridge.dc.state == DecoderState::START ||
 	       bridge.dc.state == DecoderState::DECODE);
 
-	return bridge.dc.state != DecoderState::START;
+	return bridge.dc.state == DecoderState::START
+	       ? DecodeResult::UNRECOGNIZED_FILE
+	       : DecodeResult::SUCCESS;
 }
 
 /**
@@ -132,7 +154,7 @@ decoder_stream_decode(const DecoderPlugin &plugin,
  *
  * Caller holds DecoderControl::mutex.
  */
-static bool
+static DecodeResult
 decoder_file_decode(const DecoderPlugin &plugin,
 		    DecoderBridge &bridge, Path path)
 {
@@ -146,12 +168,12 @@ decoder_file_decode(const DecoderPlugin &plugin,
 	FmtDebug(decoder_thread_domain, "probing plugin {}", plugin.name);
 
 	if (bridge.dc.command == DecoderCommand::STOP)
-		throw StopDecoder();
+		return DecodeResult::STOP;
 
 	{
 		const ScopeUnlock unlock(bridge.dc.mutex);
 
-		FormatThreadName("decoder:%s", plugin.name);
+		FmtThreadName("decoder:{}", plugin.name);
 
 		plugin.FileDecode(bridge, path);
 
@@ -161,74 +183,71 @@ decoder_file_decode(const DecoderPlugin &plugin,
 	assert(bridge.dc.state == DecoderState::START ||
 	       bridge.dc.state == DecoderState::DECODE);
 
-	return bridge.dc.state != DecoderState::START;
+	return bridge.dc.state == DecoderState::START
+	       ? DecodeResult::UNRECOGNIZED_FILE
+	       : DecodeResult::SUCCESS;
 }
 
-gcc_pure
+[[gnu::pure]]
 static bool
 decoder_check_plugin_mime(const DecoderPlugin &plugin,
 			  const InputStream &is) noexcept
 {
-	assert(plugin.stream_decode != nullptr);
-
 	const char *mime_type = is.GetMimeType();
 	return mime_type != nullptr &&
 		plugin.SupportsMimeType(GetMimeTypeBase(mime_type));
 }
 
-gcc_pure
+[[gnu::pure]]
 static bool
 decoder_check_plugin_suffix(const DecoderPlugin &plugin,
 			    std::string_view suffix) noexcept
 {
-	assert(plugin.stream_decode != nullptr);
-
 	return !suffix.empty() && plugin.SupportsSuffix(suffix);
 }
 
-gcc_pure
-static bool
-decoder_check_plugin(const DecoderPlugin &plugin, const InputStream &is,
-		     std::string_view suffix) noexcept
-{
-	return plugin.stream_decode != nullptr &&
-		(decoder_check_plugin_mime(plugin, is) ||
-		 decoder_check_plugin_suffix(plugin, suffix));
-}
-
-static bool
+static DecodeResult
 decoder_run_stream_plugin(DecoderBridge &bridge, InputStream &is,
 			  std::unique_lock<Mutex> &lock,
 			  std::string_view suffix,
-			  const DecoderPlugin &plugin,
-			  bool &tried_r)
+			  const DecoderPlugin &plugin)
 {
-	if (!decoder_check_plugin(plugin, is, suffix))
-		return false;
+	if (!decoder_check_plugin_mime(plugin, is) &&
+	    !decoder_check_plugin_suffix(plugin, suffix))
+		return DecodeResult::NO_PLUGIN;
+
+	if (plugin.stream_decode == nullptr)
+		return DecodeResult::NO_STREAM_PLUGIN;
 
 	bridge.Reset();
 
-	tried_r = true;
 	return decoder_stream_decode(plugin, bridge, is, lock);
 }
 
-static bool
+static DecodeResult
 decoder_run_stream_locked(DecoderBridge &bridge, InputStream &is,
 			  std::unique_lock<Mutex> &lock,
-			  const char *uri, bool &tried_r)
+			  std::string_view uri)
 {
 	const auto suffix = uri_get_suffix(uri);
 
-	const auto f = [&,suffix](const auto &plugin)
-		{ return decoder_run_stream_plugin(bridge, is, lock, suffix, plugin, tried_r); };
+	DecodeResult result = DecodeResult::NO_PLUGIN;
+	for (const auto &plugin : GetEnabledDecoderPlugins()) {
+		const auto r = decoder_run_stream_plugin(bridge, is, lock, suffix, plugin);
+		if (r > result) {
+			result = r;
+			if (IsFinalDecodeResult(result))
+				break;
+		}
+	}
 
-	return decoder_plugins_try(f);
+	return result;
 }
 
 /**
  * Try decoding a stream, using the fallback plugin.
  */
-static bool
+static DecodeResult
 decoder_run_stream_fallback(DecoderBridge &bridge, InputStream &is,
 			    std::unique_lock<Mutex> &lock)
 {
@@ -239,8 +258,13 @@ decoder_run_stream_fallback(DecoderBridge &bridge, InputStream &is,
 #else
 	plugin = decoder_plugin_from_name("mad");
 #endif
-	return plugin != nullptr && plugin->stream_decode != nullptr &&
-		decoder_stream_decode(*plugin, bridge, is, lock);
+	if (plugin == nullptr)
+		return DecodeResult::NO_PLUGIN;
+
+	if (plugin->stream_decode == nullptr)
+		return DecodeResult::NO_STREAM_PLUGIN;
+
+	return decoder_stream_decode(*plugin, bridge, is, lock);
 }
 
 /**
@@ -281,17 +305,27 @@ MaybeLoadReplayGain(DecoderBridge &bridge, InputStream &is)
  *
  * DecoderControl::mutex is not be locked by caller.
  */
-static bool
+static DecodeResult
 TryUriDecode(DecoderBridge &bridge, const char *uri)
 {
-	return decoder_plugins_try([&bridge, uri](const DecoderPlugin &plugin){
-		if (!plugin.SupportsUri(uri))
-			return false;
+	DecodeResult result = DecodeResult::NO_PLUGIN;
 
-		std::unique_lock<Mutex> lock(bridge.dc.mutex);
+	for (const auto &plugin : GetEnabledDecoderPlugins()) {
+		if (!plugin.SupportsUri(uri))
+			continue;
+
+		std::unique_lock lock{bridge.dc.mutex};
 		bridge.Reset();
-		return DecoderUriDecode(plugin, bridge, uri);
-	});
+
+		if (const auto r = DecoderUriDecode(plugin, bridge, uri);
+		    r > result) {
+			result = r;
+			if (IsFinalDecodeResult(result))
+				break;
+		}
+	}
+
+	return result;
 }
 
 /**
@@ -299,11 +333,12 @@ TryUriDecode(DecoderBridge &bridge, const char *uri)
  *
  * DecoderControl::mutex is not locked by caller.
  */
-static bool
+static DecodeResult
 decoder_run_stream(DecoderBridge &bridge, const char *uri)
 {
-	if (TryUriDecode(bridge, uri))
-		return true;
+	auto result = TryUriDecode(bridge, uri);
+	if (IsFinalDecodeResult(result))
+		return result;
 
 	DecoderControl &dc = bridge.dc;
 
@@ -312,16 +347,26 @@ decoder_run_stream(DecoderBridge &bridge, const char *uri)
 
 	MaybeLoadReplayGain(bridge, *input_stream);
 
-	std::unique_lock<Mutex> lock(dc.mutex);
+	std::unique_lock lock{dc.mutex};
 
-	bool tried = false;
-	return dc.command == DecoderCommand::STOP ||
-		decoder_run_stream_locked(bridge, *input_stream, lock, uri,
-					  tried) ||
-		/* fallback to mp3: this is needed for bastard streams
-		   that don't have a suffix or set the mimeType */
-		(!tried &&
-		 decoder_run_stream_fallback(bridge, *input_stream, lock));
+	if (dc.command == DecoderCommand::STOP)
+		return DecodeResult::STOP;
+
+	if (auto r = decoder_run_stream_locked(bridge, *input_stream, lock, uri);
+	    r > result) {
+		result = r;
+		if (IsFinalDecodeResult(result))
+			return result;
+	}
+
+	/* fallback to mp3: this is needed for bastard streams that
+	   don't have a suffix or set the mimeType */
+	if (auto r = decoder_run_stream_fallback(bridge, *input_stream, lock);
+	    r > result) {
+		result = r;
+	}
+
+	return result;
 }
 
 /**
@@ -329,27 +374,27 @@ decoder_run_stream(DecoderBridge &bridge, const char *uri)
  *
  * DecoderControl::mutex is not locked by caller.
  */
-static bool
+static DecodeResult
 TryDecoderFile(DecoderBridge &bridge, Path path_fs, std::string_view suffix,
 	       InputStream &input_stream,
 	       const DecoderPlugin &plugin)
 {
 	if (!plugin.SupportsSuffix(suffix))
-		return false;
+		return DecodeResult::NO_PLUGIN;
 
 	bridge.Reset();
 
 	DecoderControl &dc = bridge.dc;
 
 	if (plugin.file_decode != nullptr) {
-		const std::scoped_lock<Mutex> protect(dc.mutex);
+		const std::scoped_lock protect{dc.mutex};
 		return decoder_file_decode(plugin, bridge, path_fs);
 	} else if (plugin.stream_decode != nullptr) {
-		std::unique_lock<Mutex> lock(dc.mutex);
+		std::unique_lock lock{dc.mutex};
 		return decoder_stream_decode(plugin, bridge, input_stream,
 					     lock);
 	} else
-		return false;
+		return DecodeResult::NO_STREAM_PLUGIN;
 }
 
 /**
@@ -357,7 +402,7 @@ TryDecoderFile(DecoderBridge &bridge, Path path_fs, std::string_view suffix,
  *
  * DecoderControl::mutex is not locked by caller.
  */
-static bool
+static DecodeResult
 TryContainerDecoder(DecoderBridge &bridge, Path path_fs,
 		    std::string_view suffix,
 		    const DecoderPlugin &plugin)
@@ -365,12 +410,12 @@ TryContainerDecoder(DecoderBridge &bridge, Path path_fs,
 	if (plugin.container_scan == nullptr ||
 	    plugin.file_decode == nullptr ||
 	    !plugin.SupportsSuffix(suffix))
-		return false;
+		return DecodeResult::NO_PLUGIN;
 
 	bridge.Reset();
 
 	DecoderControl &dc = bridge.dc;
-	const std::scoped_lock<Mutex> protect(dc.mutex);
+	const std::scoped_lock protect{dc.mutex};
 	return decoder_file_decode(plugin, bridge, path_fs);
 }
 
@@ -379,17 +424,22 @@ TryContainerDecoder(DecoderBridge &bridge, Path path_fs,
  *
  * DecoderControl::mutex is not locked by caller.
  */
-static bool
+static DecodeResult
 TryContainerDecoder(DecoderBridge &bridge, Path path_fs,
 		    std::string_view suffix)
 {
-	return decoder_plugins_try([&bridge, path_fs,
-				    suffix](const DecoderPlugin &plugin){
-					   return TryContainerDecoder(bridge,
-								      path_fs,
-								      suffix,
-								      plugin);
-				   });
+	DecodeResult result = DecodeResult::NO_PLUGIN;
+
+	for (const auto &plugin : GetEnabledDecoderPlugins()) {
+		if (const auto r = TryContainerDecoder(bridge, path_fs, suffix, plugin);
+		    r > result) {
+			result = r;
+			if (IsFinalDecodeResult(result))
+				break;
+		}
+	}
+
+	return result;
 }
 
 /**
@@ -397,12 +447,12 @@ TryContainerDecoder(DecoderBridge &bridge, Path path_fs,
  *
  * DecoderControl::mutex is not locked by caller.
  */
-static bool
+static DecodeResult
 decoder_run_file(DecoderBridge &bridge, const char *uri_utf8, Path path_fs)
 {
 	const char *_suffix = PathTraitsUTF8::GetFilenameSuffix(uri_utf8);
 	if (_suffix == nullptr)
-		return false;
+		return DecodeResult::NO_PLUGIN;
 
 	const std::string_view suffix{_suffix};
 
@@ -411,11 +461,13 @@ decoder_run_file(DecoderBridge &bridge, const char *uri_utf8, Path path_fs)
 	try {
 		input_stream = bridge.OpenLocal(path_fs, uri_utf8);
 	} catch (const std::system_error &e) {
-		if (IsPathNotFound(e) &&
+		if (IsPathNotFound(e)) {
 		    /* ENOTDIR means this may be a path inside a
 		       "container" file */
-		    TryContainerDecoder(bridge, path_fs, suffix))
-			return true;
+			const auto result = TryContainerDecoder(bridge, path_fs, suffix);
+			if (IsFinalDecodeResult(result))
+				return result;
+		}
 
 		throw;
 	}
@@ -424,15 +476,17 @@ decoder_run_file(DecoderBridge &bridge, const char *uri_utf8, Path path_fs)
 
 	MaybeLoadReplayGain(bridge, *input_stream);
 
-	auto &is = *input_stream;
-	return decoder_plugins_try([&bridge, path_fs, suffix,
-				    &is](const DecoderPlugin &plugin){
-					   return TryDecoderFile(bridge,
-								 path_fs,
-								 suffix,
-								 is,
-								 plugin);
-				   });
+	DecodeResult result = DecodeResult::NO_PLUGIN;
+	for (const auto &plugin : GetEnabledDecoderPlugins()) {
+		if (const auto r = TryDecoderFile(bridge, path_fs, suffix, *input_stream, plugin);
+		    r > result) {
+			result = r;
+			if (IsFinalDecodeResult(result))
+				break;
+		}
+	}
+
+	return result;
 }
 
 /**
@@ -440,7 +494,7 @@ decoder_run_file(DecoderBridge &bridge, const char *uri_utf8, Path path_fs)
  *
  * DecoderControl::mutex is not locked.
  */
-static bool
+static DecodeResult
 DecoderUnlockedRunUri(DecoderBridge &bridge,
 		      const char *real_uri, Path path_fs)
 try {
@@ -448,15 +502,15 @@ try {
 		? decoder_run_file(bridge, real_uri, path_fs)
 		: decoder_run_stream(bridge, real_uri);
 } catch (StopDecoder) {
-	return true;
+	return DecodeResult::STOP;
 } catch (...) {
 	const char *error_uri = real_uri;
 	const std::string allocated = uri_remove_auth(error_uri);
 	if (!allocated.empty())
 		error_uri = allocated.c_str();
 
-	std::throw_with_nested(FormatRuntimeError("Failed to decode %s",
-						  error_uri));
+	std::throw_with_nested(FmtRuntimeError("Failed to decode {:?}",
+					       error_uri));
 }
 
 /**
@@ -465,13 +519,25 @@ try {
  * are only kept as a cache to be displayed by the client; they shall
  * not be sent to the output.
  */
-gcc_pure
+[[gnu::pure]]
 static bool
 SongHasVolatileTags(const DetachedSong &song) noexcept
 {
 	return !song.IsFile() && !HasRemoteTagScanner(song.GetRealURI());
 }
 
+[[gnu::pure]]
+static std::runtime_error
+MakeDecoderError(const DetachedSong &song, const char *msg) noexcept
+{
+	const char *error_uri = song.GetURI();
+	const std::string allocated = uri_remove_auth(error_uri);
+	if (!allocated.empty())
+		error_uri = allocated.c_str();
+
+	return FmtRuntimeError("Failed to decode {:?}: {}", error_uri, msg);
+}
+
 /**
  * Decode a song addressed by a #DetachedSong.
  *
@@ -498,7 +564,7 @@ decoder_run_song(DecoderControl &dc,
 	dc.state = DecoderState::START;
 	dc.CommandFinishedLocked();
 
-	bool success;
+	DecodeResult result;
 	{
 		const ScopeUnlock unlock(dc.mutex);
 
@@ -507,21 +573,25 @@ decoder_run_song(DecoderControl &dc,
 			bridge.CheckFlushChunk();
 		};
 
-		success = DecoderUnlockedRunUri(bridge, uri, path_fs);
-
+		result = DecoderUnlockedRunUri(bridge, uri, path_fs);
 	}
 
 	bridge.CheckRethrowError();
 
-	if (success)
-		dc.state = DecoderState::STOP;
-	else {
-		const char *error_uri = song.GetURI();
-		const std::string allocated = uri_remove_auth(error_uri);
-		if (!allocated.empty())
-			error_uri = allocated.c_str();
+	switch (result) {
+	case DecodeResult::NO_PLUGIN:
+		throw MakeDecoderError(song, "no decoder plugin");
 
-		throw FormatRuntimeError("Failed to decode %s", error_uri);
+	case DecodeResult::NO_STREAM_PLUGIN:
+		throw MakeDecoderError(song, "no streaming decoder plugin");
+
+	case DecodeResult::UNRECOGNIZED_FILE:
+		throw MakeDecoderError(song, "unrecognized file");
+
+	case DecodeResult::STOP:
+	case DecodeResult::SUCCESS:
+		dc.state = DecoderState::STOP;
+		break;
 	}
 
 	dc.client_cond.notify_one();
@@ -561,7 +631,7 @@ DecoderControl::RunThread() noexcept
 {
 	SetThreadName("decoder");
 
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 
 	do {
 		assert(state == DecoderState::STOP ||
