@@ -112,6 +112,12 @@ flac_decoder_new() noexcept
 		LogDebug(flac_domain,
 			 "FLAC__stream_decoder_set_metadata_respond() has failed");
 
+#if FLAC_API_VERSION_CURRENT >= 14
+	if (!FLAC__stream_decoder_set_decode_chained_stream(sd.get(), true))
+		LogDebug(flac_domain,
+			 "FLAC__stream_decoder_set_decode_chained_stream() has failed");
+#endif
+
 	return sd;
 }
 
@@ -222,8 +228,12 @@ flac_decoder_loop(FlacDecoder *data, FLAC__StreamDecoder *flac_dec)
 
 #if FLAC_API_VERSION_CURRENT >= 14
 		case FLAC__STREAM_DECODER_END_OF_LINK:
-			/* TODO support FLAC 1.5 chained streams */
-			return;
+			if (!FLAC__stream_decoder_finish_link(flac_dec)) {
+				LogError(flac_domain, "FLAC__stream_decoder_finish_link() failed");
+				return;
+			}
+
+			break;
 #endif
 		}
 
