@@ -1,31 +1,16 @@
-/*
- * Copyright 2003-2020 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "WildmidiDecoderPlugin.hxx"
 #include "../DecoderAPI.hxx"
 #include "tag/Handler.hxx"
 #include "util/ScopeExit.hxx"
-#include "util/StringFormat.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/FileSystem.hxx"
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
+#include "lib/fmt/ToBuffer.hxx"
+#include "lib/fmt/PathFormatter.hxx"
 #include "PluginUnavailable.hxx"
 
 #ifdef _WIN32
@@ -47,11 +32,9 @@ wildmidi_init(const ConfigBlock &block)
 		block.GetPath("config_file",
 			      "/etc/timidity/timidity.cfg");
 
-	if (!FileExists(path)) {
-		const auto utf8 = path.ToUTF8();
-		throw PluginUnavailable(StringFormat<1024>("configuration file does not exist: %s",
-							   utf8.c_str()));
-	}
+	if (!FileExists(path))
+		throw PluginUnavailable{FmtBuffer<1024>("configuration file does not exist: {}",
+							path)};
 
 #ifdef LIBWILDMIDI_VERSION
 	/* WildMidi_ClearError() requires libwildmidi 0.4 */
@@ -94,7 +77,9 @@ wildmidi_output(DecoderClient &client, midi *wm)
 	if (length <= 0)
 		return DecoderCommand::STOP;
 
-	return client.SubmitData(nullptr, buffer, length, 0);
+	return client.SubmitAudio(nullptr,
+				  std::span{buffer, std::size_t(length)},
+				  0);
 }
 
 static void
