@@ -1,25 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
-#include "config.h"
 #include "ServerSocket.hxx"
 #include "lib/fmt/ExceptionFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
+#include "lib/fmt/SocketAddressFormatter.hxx"
 #include "net/IPv4Address.hxx"
 #include "net/IPv6Address.hxx"
 #include "net/StaticSocketAddress.hxx"
@@ -29,14 +14,11 @@
 #include "net/UniqueSocketDescriptor.hxx"
 #include "net/Resolver.hxx"
 #include "net/AddressInfo.hxx"
-#include "net/ToString.hxx"
 #include "event/SocketEvent.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "util/RuntimeError.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
-#include <cassert>
 #include <string>
 #include <utility>
 
@@ -91,7 +73,7 @@ public:
 	}
 #endif
 
-	bool IsDefined() const noexcept {
+	[[nodiscard]] bool IsDefined() const noexcept {
 		return event.IsDefined();
 	}
 
@@ -101,9 +83,9 @@ public:
 		event.Close();
 	}
 
-	[[nodiscard]] [[gnu::pure]]
-	std::string ToString() const noexcept {
-		return ::ToString(address);
+	[[nodiscard]]
+	SocketAddress GetAddress() const noexcept {
+		return address;
 	}
 
 	void SetFD(UniqueSocketDescriptor _fd) noexcept {
@@ -119,36 +101,11 @@ private:
 
 static constexpr Domain server_socket_domain("server_socket");
 
-static int
-get_remote_uid(int fd)
-{
-#ifdef HAVE_STRUCT_UCRED
-	struct ucred cred;
-	socklen_t len = sizeof (cred);
-
-	if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cred, &len) < 0)
-		return -1;
-
-	return cred.uid;
-#else
-#ifdef HAVE_GETPEEREID
-	uid_t euid;
-	gid_t egid;
-
-	if (getpeereid(fd, &euid, &egid) == 0)
-		return euid;
-#else
-	(void)fd;
-#endif
-	return -1;
-#endif
-}
-
 inline void
 ServerSocket::OneServerSocket::Accept() noexcept
 {
 	StaticSocketAddress peer_address;
-	UniqueSocketDescriptor peer_fd(event.GetSocket().AcceptNonBlock(peer_address));
+	UniqueSocketDescriptor peer_fd{AdoptTag{}, event.GetSocket().AcceptNonBlock(peer_address)};
 	if (!peer_fd.IsDefined()) {
 		const SocketErrorMessage msg;
 		FmtError(server_socket_domain,
@@ -163,9 +120,7 @@ ServerSocket::OneServerSocket::Accept() noexcept
 			 (const char *)msg);
 	}
 
-	const auto uid = get_remote_uid(peer_fd.Get());
-
-	parent.OnAccept(std::move(peer_fd), peer_address, uid);
+	parent.OnAccept(std::move(peer_fd), peer_address);
 }
 
 void
@@ -183,6 +138,22 @@ ServerSocket::OneServerSocket::Open()
 				      SOCK_STREAM, 0,
 				      address, 5);
 
+#ifdef HAVE_TCP
+	if (parent.dscp_class >= 0) {
+		const int family = address.GetFamily();
+		if ((family == AF_INET &&
+		     !_fd.SetIntOption(IPPROTO_IP, IP_TOS, parent.dscp_class)) ||
+		    (family == AF_INET6 &&
+		     !_fd.SetIntOption(IPPROTO_IPV6, IPV6_TCLASS,
+				       parent.dscp_class))) {
+			const SocketErrorMessage msg;
+			FmtError(server_socket_domain,
+				 "Could not set DSCP class: {}",
+				 (const char *)msg);
+		}
+	}
+#endif
+
 #ifdef HAVE_UN
 	/* allow everybody to connect */
 
@@ -218,7 +189,6 @@ ServerSocket::Open()
 			continue;
 
 		if (bad != nullptr && i.GetSerial() != bad->GetSerial()) {
-			Close();
 			std::rethrow_exception(last_error);
 		}
 
@@ -226,23 +196,19 @@ ServerSocket::Open()
 			i.Open();
 		} catch (...) {
 			if (good != nullptr && good->GetSerial() == i.GetSerial()) {
-				const auto address_string = i.ToString();
-				const auto good_string = good->ToString();
 				FmtError(server_socket_domain,
 					 "bind to '{}' failed "
 					 "(continuing anyway, because "
 					 "binding to '{}' succeeded): {}",
-					 address_string,
-					 good_string,
+					 i.GetAddress(),
+					 good->GetAddress(),
 					 std::current_exception());
 			} else if (bad == nullptr) {
 				bad = &i;
 
-				const auto address_string = i.ToString();
-
 				try {
-					std::throw_with_nested(FormatRuntimeError("Failed to bind to '%s'",
-										  address_string.c_str()));
+					std::throw_with_nested(FmtRuntimeError("Failed to bind to '{}'",
+									       i.GetAddress()));
 				} catch (...) {
 					last_error = std::current_exception();
 				}
@@ -263,7 +229,6 @@ ServerSocket::Open()
 	}
 
 	if (bad != nullptr) {
-		Close();
 		std::rethrow_exception(last_error);
 	}
 }
@@ -334,12 +299,7 @@ ServerSocket::AddPortIPv6(unsigned port) noexcept
 static bool
 SupportsIPv6() noexcept
 {
-	int fd = socket(AF_INET6, SOCK_STREAM, 0);
-	if (fd < 0)
-		return false;
-
-	close(fd);
-	return true;
+	return UniqueSocketDescriptor{}.Create(AF_INET6, SOCK_STREAM, 0);
 }
 
 #endif /* HAVE_IPV6 */
