@@ -1,28 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "IcyInputStream.hxx"
 #include "tag/IcyMetaDataParser.hxx"
 #include "tag/Tag.hxx"
 #include "util/UriExtract.hxx"
 #include "util/UriQueryParser.hxx"
-#include "util/StringView.hxx"
 
 #include <string>
 
@@ -35,10 +18,8 @@ IcyInputStream::IcyInputStream(InputStreamPtr _input,
 	if (fragment != nullptr) {
 		const auto charset = UriFindRawQueryParameter(fragment,
 							      "charset");
-		if (charset != nullptr) {
-			const std::string copy(charset.data, charset.size);
-			parser->SetCharset(copy.c_str());
-		}
+		if (charset.data() != nullptr)
+			parser->SetCharset(std::string{charset}.c_str());
 	}
 #endif
 }
@@ -87,20 +68,20 @@ IcyInputStream::ReadTag() noexcept
 
 size_t
 IcyInputStream::Read(std::unique_lock<Mutex> &lock,
-		     void *ptr, size_t read_size)
+		     std::span<std::byte> dest)
 {
 	if (!IsEnabled())
-		return ProxyInputStream::Read(lock, ptr, read_size);
+		return ProxyInputStream::Read(lock, dest);
 
 	while (true) {
-		size_t nbytes = ProxyInputStream::Read(lock, ptr, read_size);
+		size_t nbytes = ProxyInputStream::Read(lock, dest);
 		if (nbytes == 0) {
 			assert(IsEOF());
 			offset = override_offset;
 			return 0;
 		}
 
-		size_t result = parser->ParseInPlace(ptr, nbytes);
+		size_t result = parser->ParseInPlace(dest.first(nbytes));
 		if (result > 0) {
 			override_offset += result;
 			offset = override_offset;
