@@ -1,28 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "DetectFilterFormat.hxx"
 #include "Frame.hxx"
 #include "SampleFormat.hxx"
 #include "pcm/Silence.hxx"
 #include "pcm/CheckAudioFormat.hxx"
-#include "util/WritableBuffer.hxx"
 
 extern "C" {
 #include <libavfilter/buffersrc.h>
@@ -42,13 +25,17 @@ DetectFilterOutputFormat(const AudioFormat &in_audio_format,
 	const size_t silence_size = in_audio_format.GetFrameSize();
 	assert(sizeof(silence) >= silence_size);
 
-	PcmSilence(WritableBuffer<void>{&silence, silence_size},
+	PcmSilence(std::as_writable_bytes(std::span{silence}),
 		   in_audio_format.format);
 
 	Frame frame;
 	frame->format = ToFfmpegSampleFormat(in_audio_format.format);
 	frame->sample_rate = in_audio_format.sample_rate;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 25, 100)
+	av_channel_layout_default(&frame->ch_layout, in_audio_format.channels);
+#else
 	frame->channels = in_audio_format.channels;
+#endif
 	frame->nb_samples = 1;
 
 	frame.GetBuffer();
@@ -75,8 +62,14 @@ DetectFilterOutputFormat(const AudioFormat &in_audio_format,
 	if (sample_format == SampleFormat::UNDEFINED)
 		throw std::runtime_error("Unsupported FFmpeg sample format");
 
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 25, 100)
+	const unsigned out_channels = frame->ch_layout.nb_channels;
+#else
+	const unsigned out_channels = frame->channels;
+#endif
+
 	return CheckAudioFormat(frame->sample_rate, sample_format,
-				frame->channels);
+				out_channels);
 }
 
 } // namespace Ffmpeg
