@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Device.hxx"
 #include "Util.hxx"
@@ -23,9 +7,6 @@
 
 #include <string.h>
 
-/* this destructor exists here just so it won't get inlined */
-UPnPDevice::~UPnPDevice() noexcept = default;
-
 /**
  * An XML parser which constructs an UPnP device object from the
  * device descriptor.
@@ -83,23 +64,23 @@ protected:
 			trimstring(*value);
 			value = nullptr;
 		} else if (!strcmp(name, "service")) {
-			m_device.services.emplace_back(std::move(m_tservice));
+			m_device.services.emplace_front(std::move(m_tservice));
 			m_tservice = {};
 		}
 	}
 
-	void CharacterData(const XML_Char *s, int len) override {
+	void CharacterData(std::string_view s) override {
 		if (value != nullptr)
-			value->append(s, len);
+			value->append(s);
 	}
 };
 
 void
-UPnPDevice::Parse(const std::string &url, const char *description)
+UPnPDevice::Parse(const std::string_view url, const std::string_view description)
 {
 	{
 		UPnPDeviceParser mparser(*this);
-		mparser.Parse(description, strlen(description), true);
+		mparser.Parse(description, true);
 	}
 
 	if (URLBase.empty()) {
