@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "VorbisComments.hxx"
 #include "VorbisPicture.hxx"
@@ -24,9 +8,8 @@
 #include "tag/Builder.hxx"
 #include "tag/Tag.hxx"
 #include "tag/VorbisComment.hxx"
-#include "tag/ReplayGain.hxx"
-#include "ReplayGainInfo.hxx"
-#include "util/StringView.hxx"
+#include "tag/ReplayGainInfo.hxx"
+#include "tag/ReplayGainParser.hxx"
 #include "decoder/Features.h"
 
 #ifndef HAVE_TREMOR
@@ -44,7 +27,7 @@ ForEachUserComment(const vorbis_comment &vc, F &&f)
 
 	const size_t n = vc.comments;
 	for (size_t i = 0; i < n; ++i)
-		f(StringView{user_comments[i], size_t(comment_lengths[i])});
+		f(std::string_view{user_comments[i], size_t(comment_lengths[i])});
 }
 
 bool
@@ -55,8 +38,8 @@ VorbisCommentToReplayGain(ReplayGainInfo &rgi,
 
 	bool found = false;
 
-	ForEachUserComment(vc, [&](StringView s){
-		if (ParseReplayGainVorbis(rgi, s.data))
+	ForEachUserComment(vc, [&](std::string_view s){
+		if (ParseReplayGainVorbis(rgi, s))
 			found = true;
 	});
 
@@ -64,12 +47,12 @@ VorbisCommentToReplayGain(ReplayGainInfo &rgi,
 }
 
 static void
-vorbis_scan_comment(StringView comment, TagHandler &handler) noexcept
+vorbis_scan_comment(std::string_view comment, TagHandler &handler) noexcept
 {
 	const auto picture_b64 = handler.WantPicture()
 		? GetVorbisCommentValue(comment, "METADATA_BLOCK_PICTURE")
-		: nullptr;
-	if (!picture_b64.IsNull())
+		: std::string_view{};
+	if (picture_b64.data() != nullptr)
 		return ScanVorbisPicture(picture_b64, handler);
 
 	ScanVorbisComment(comment, handler);
@@ -78,7 +61,7 @@ vorbis_scan_comment(StringView comment, TagHandler &handler) noexcept
 void
 VorbisCommentScan(const vorbis_comment &vc, TagHandler &handler) noexcept
 {
-	ForEachUserComment(vc, [&](StringView s){
+	ForEachUserComment(vc, [&](std::string_view s){
 		vorbis_scan_comment(s, handler);
 	});
 }
