@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "UdisksNeighborPlugin.hxx"
 #include "lib/dbus/Connection.hxx"
@@ -67,9 +51,9 @@ class UdisksNeighborExplorer final
 	 */
 	mutable Mutex mutex;
 
-	using ByUri = std::map<std::string, NeighborInfo>;
+	using ByUri = std::map<std::string, NeighborInfo, std::less<>>;
 	ByUri by_uri;
-	std::map<std::string, ByUri::iterator> by_path;
+	std::map<std::string, ByUri::iterator, std::less<>> by_path;
 
 public:
 	UdisksNeighborExplorer(EventLoop &_event_loop,
@@ -175,7 +159,7 @@ UdisksNeighborExplorer::Close() noexcept
 NeighborExplorer::List
 UdisksNeighborExplorer::GetList() const noexcept
 {
-	const std::lock_guard<Mutex> lock(mutex);
+	const std::scoped_lock lock{mutex};
 
 	NeighborExplorer::List result;
 
@@ -192,7 +176,7 @@ UdisksNeighborExplorer::Insert(UDisks2::Object &&o) noexcept
 	const NeighborInfo info = ToNeighborInfo(o);
 
 	{
-		const std::lock_guard<Mutex> protect(mutex);
+		const std::scoped_lock protect{mutex};
 		auto i = by_uri.emplace(o.GetUri(), info);
 		if (!i.second)
 			i.first->second = info;
@@ -207,7 +191,7 @@ UdisksNeighborExplorer::Insert(UDisks2::Object &&o) noexcept
 void
 UdisksNeighborExplorer::Remove(const std::string &path) noexcept
 {
-	std::unique_lock<Mutex> lock(mutex);
+	std::unique_lock lock{mutex};
 
 	auto i = by_path.find(path);
 	if (i == by_path.end())
