@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "RecorderOutputPlugin.hxx"
 #include "../OutputAPI.hxx"
@@ -86,9 +70,9 @@ private:
 
 	void SendTag(const Tag &tag) override;
 
-	size_t Play(const void *chunk, size_t size) override;
+	std::size_t Play(std::span<const std::byte> src) override;
 
-	[[nodiscard]] gcc_pure
+	[[nodiscard]] [[gnu::pure]]
 	bool HasDynamicPath() const noexcept {
 		return !format_path.empty();
 	}
@@ -277,7 +261,7 @@ RecorderOutput::ReopenFormat(AllocatedPath &&new_path)
 	path = std::move(new_path);
 	file = new_file;
 
-	FmtDebug(recorder_domain, "Recording to \"{}\"", path);
+	FmtDebug(recorder_domain, "Recording to {:?}", path);
 }
 
 void
@@ -322,22 +306,22 @@ RecorderOutput::SendTag(const Tag &tag)
 	encoder->SendTag(tag);
 }
 
-size_t
-RecorderOutput::Play(const void *chunk, size_t size)
+std::size_t
+RecorderOutput::Play(std::span<const std::byte> src)
 {
 	if (file == nullptr) {
 		/* not currently encoding to a file; discard incoming
 		   data */
 		assert(HasDynamicPath());
 		assert(path.IsNull());
-		return size;
+		return src.size();
 	}
 
-	encoder->Write(chunk, size);
+	encoder->Write(src);
 
 	EncoderToFile();
 
-	return size;
+	return src.size();
 }
 
 const struct AudioOutputPlugin recorder_output_plugin = {
