@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2020 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /** \file
  *
@@ -33,12 +17,14 @@
 #include "TagFile.hxx"
 #include "fs/Traits.hxx"
 #include "fs/AllocatedPath.hxx"
-#include "util/StringView.hxx"
+#include "util/StringCompare.hxx"
 
 #include <memory>
 
 #include <string.h>
 
+using std::string_view_literals::operator""sv;
+
 class EmbeddedCuePlaylist final : public SongEnumerator {
 public:
 	/**
@@ -69,14 +55,14 @@ public:
 
 	ExtractCuesheetTagHandler() noexcept:NullTagHandler(WANT_PAIR) {}
 
-	void OnPair(StringView key, StringView value) noexcept override;
+	void OnPair(std::string_view key, std::string_view value) noexcept override;
 };
 
 void
-ExtractCuesheetTagHandler::OnPair(StringView name, StringView value) noexcept
+ExtractCuesheetTagHandler::OnPair(std::string_view name, std::string_view value) noexcept
 {
-	if (cuesheet.empty() && name.EqualsIgnoreCase("cuesheet"))
-		cuesheet = {value.data, value.size};
+	if (cuesheet.empty() && StringIsEqualIgnoreCase(name, "cuesheet"sv))
+		cuesheet = value;
 }
 
 static std::unique_ptr<SongEnumerator>
@@ -104,7 +90,7 @@ embcue_playlist_open_uri(const char *uri,
 
 	playlist->cuesheet = std::move(extract_cuesheet.cuesheet);
 
-	playlist->next = &playlist->cuesheet[0];
+	playlist->next = playlist->cuesheet.data();
 	playlist->parser = std::make_unique<CueParser>();
 
 	return playlist;
