@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 /*
  * Saving and loading the playlist to/from the state file.
@@ -27,16 +11,18 @@
 #include "Playlist.hxx"
 #include "SingleMode.hxx"
 #include "StateFileConfig.hxx"
-#include "queue/QueueSave.hxx"
-#include "fs/io/TextFile.hxx"
-#include "fs/io/BufferedOutputStream.hxx"
+#include "Save.hxx"
+#include "io/LineReader.hxx"
+#include "io/BufferedOutputStream.hxx"
 #include "player/Control.hxx"
 #include "util/CharUtil.hxx"
 #include "util/StringAPI.hxx"
 #include "util/StringCompare.hxx"
-#include "util/NumberParser.hxx"
+#include "util/CNumberParser.hxx"
 #include "Log.hxx"
 
+#include <fmt/format.h>
+
 #include <string.h>
 #include <stdlib.h>
 
@@ -50,6 +36,7 @@
 #define PLAYLIST_STATE_FILE_CROSSFADE		"crossfade: "
 #define PLAYLIST_STATE_FILE_MIXRAMPDB		"mixrampdb: "
 #define PLAYLIST_STATE_FILE_MIXRAMPDELAY	"mixrampdelay: "
+#define PLAYLIST_STATE_FILE_LOADED_PLAYLIST	"lastloadedplaylist: "
 #define PLAYLIST_STATE_FILE_PLAYLIST_BEGIN	"playlist_begin"
 #define PLAYLIST_STATE_FILE_PLAYLIST_END	"playlist_end"
 
@@ -73,36 +60,41 @@ playlist_state_save(BufferedOutputStream &os, const struct playlist &playlist,
 		default:
 			os.Write(PLAYLIST_STATE_FILE_STATE_PLAY "\n");
 		}
-		os.Format(PLAYLIST_STATE_FILE_CURRENT "%i\n",
-			  playlist.queue.OrderToPosition(playlist.current));
-		os.Format(PLAYLIST_STATE_FILE_TIME "%f\n",
-			  player_status.elapsed_time.ToDoubleS());
+		os.Fmt(PLAYLIST_STATE_FILE_CURRENT "{}\n",
+		       playlist.queue.OrderToPosition(playlist.current));
+		os.Fmt(PLAYLIST_STATE_FILE_TIME "{}\n",
+		       player_status.elapsed_time.ToDoubleS());
 	} else {
 		os.Write(PLAYLIST_STATE_FILE_STATE_STOP "\n");
 
 		if (playlist.current >= 0)
-			os.Format(PLAYLIST_STATE_FILE_CURRENT "%i\n",
-				playlist.queue.OrderToPosition(playlist.current));
+			os.Fmt(PLAYLIST_STATE_FILE_CURRENT "{}\n",
+			       playlist.queue.OrderToPosition(playlist.current));
 	}
 
-	os.Format(PLAYLIST_STATE_FILE_RANDOM "%i\n", playlist.queue.random);
-	os.Format(PLAYLIST_STATE_FILE_REPEAT "%i\n", playlist.queue.repeat);
-	os.Format(PLAYLIST_STATE_FILE_SINGLE "%i\n",
-			  (int)playlist.queue.single);
-	os.Format(PLAYLIST_STATE_FILE_CONSUME "%i\n", playlist.queue.consume);
-	os.Format(PLAYLIST_STATE_FILE_CROSSFADE "%i\n",
-		  (int)pc.GetCrossFade().count());
-	os.Format(PLAYLIST_STATE_FILE_MIXRAMPDB "%f\n",
-		  (double)pc.GetMixRampDb());
-	os.Format(PLAYLIST_STATE_FILE_MIXRAMPDELAY "%f\n",
-		  pc.GetMixRampDelay().count());
+	os.Fmt(PLAYLIST_STATE_FILE_RANDOM "{}\n",
+	       (unsigned)playlist.queue.random);
+	os.Fmt(PLAYLIST_STATE_FILE_REPEAT "{}\n",
+	       (unsigned)playlist.queue.repeat);
+	os.Fmt(PLAYLIST_STATE_FILE_SINGLE "{}\n",
+		   SingleToString(playlist.queue.single));
+	os.Fmt(PLAYLIST_STATE_FILE_CONSUME "{}\n",
+	       ConsumeToString(playlist.queue.consume));
+	os.Fmt(PLAYLIST_STATE_FILE_CROSSFADE "{}\n",
+	       pc.GetCrossFade().count());
+	os.Fmt(PLAYLIST_STATE_FILE_MIXRAMPDB "{}\n",
+	       pc.GetMixRampDb());
+	os.Fmt(PLAYLIST_STATE_FILE_MIXRAMPDELAY "{}\n",
+	       pc.GetMixRampDelay().count());
+	os.Fmt(PLAYLIST_STATE_FILE_LOADED_PLAYLIST "{}\n",
+	       playlist.GetLastLoadedPlaylist());
 	os.Write(PLAYLIST_STATE_FILE_PLAYLIST_BEGIN "\n");
 	queue_save(os, playlist.queue);
 	os.Write(PLAYLIST_STATE_FILE_PLAYLIST_END "\n");
 }
 
 static void
-playlist_state_load(TextFile &file, const SongLoader &song_loader,
+playlist_state_load(LineReader &file, const SongLoader &song_loader,
 		    struct playlist &playlist)
 {
 	const char *line = file.ReadLine();
@@ -128,7 +120,7 @@ playlist_state_load(TextFile &file, const SongLoader &song_loader,
 
 bool
 playlist_state_restore(const StateFileConfig &config,
-		       const char *line, TextFile &file,
+		       const char *line, LineReader &file,
 		       const SongLoader &song_loader,
 		       struct playlist &playlist, PlayerControl &pc)
 {
@@ -157,7 +149,7 @@ playlist_state_restore(const StateFileConfig &config,
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_SINGLE))) {
 			playlist.SetSingle(pc, SingleFromString(p));
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_CONSUME))) {
-			playlist.SetConsume(StringIsEqual(p, "1"));
+			playlist.SetConsume(ConsumeFromString(p));
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_CROSSFADE))) {
 			pc.SetCrossFade(FloatDuration(atoi(p)));
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_MIXRAMPDB))) {
@@ -167,6 +159,8 @@ playlist_state_restore(const StateFileConfig &config,
 			   prior to MPD 0.18 */
 			if (IsDigitASCII(*p))
 				pc.SetMixRampDelay(FloatDuration(ParseFloat(p)));
+		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_LOADED_PLAYLIST))) {
+			playlist.SetLastLoadedPlaylist(p);
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_RANDOM))) {
 			random_mode = StringIsEqual(p, "1");
 		} else if ((p = StringAfterPrefix(line, PLAYLIST_STATE_FILE_CURRENT))) {
@@ -238,6 +232,7 @@ playlist_state_get_hash(const playlist &playlist,
 		((int)playlist.queue.single << 25) ^
 		(playlist.queue.random << 27) ^
 		(playlist.queue.repeat << 28) ^
-		(playlist.queue.consume << 30) ^
+		/* note that this takes 2 bits */
+		((int)playlist.queue.consume << 29) ^
 		(playlist.queue.random << 31);
 }
