@@ -1,28 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ApeTag.hxx"
 #include "ApeLoader.hxx"
 #include "ParseName.hxx"
 #include "Table.hxx"
 #include "Handler.hxx"
-#include "util/StringView.hxx"
 #include "util/IterableSplitString.hxx"
 
 static constexpr struct tag_table ape_tags[] = {
@@ -46,7 +29,7 @@ tag_ape_name_parse(const char *name)
  */
 static bool
 tag_ape_import_item(unsigned long flags,
-		    const char *key, StringView value,
+		    const char *key, std::string_view value,
 		    TagHandler &handler) noexcept
 {
 	/* we only care about utf-8 text tags */
@@ -74,7 +57,7 @@ tag_ape_scan2(InputStream &is, TagHandler &handler)
 
 	auto callback = [&handler, &recognized]
 		(unsigned long flags, const char *key,
-		 StringView value) {
+		 std::string_view value) {
 		recognized |= tag_ape_import_item(flags, key, value, handler);
 		return true;
 	};
