@@ -1,31 +1,5 @@
-/*
- * Copyright 2011-2021 Max Kellermann <max.kellermann@gmail.com>
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * - Redistributions of source code must retain the above copyright
- * notice, this list of conditions and the following disclaimer.
- *
- * - Redistributions in binary form must reproduce the above copyright
- * notice, this list of conditions and the following disclaimer in the
- * documentation and/or other materials provided with the
- * distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
- * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+// author: Max Kellermann <max.kellermann@gmail.com>
 
 #include "UTF8.hxx"
 #include "CharUtil.hxx"
@@ -298,38 +272,38 @@ Latin1ToUTF8(const char *gcc_restrict src, char *gcc_restrict buffer,
 char *
 UnicodeToUTF8(unsigned ch, char *q) noexcept
 {
-  if (gcc_likely(ch < 0x80)) {
-    *q++ = (char)ch;
-  } else if (gcc_likely(ch < 0x800)) {
-    *q++ = MakeLeading1(ch >> 6);
-    *q++ = MakeContinuation(ch);
-  } else if (ch < 0x10000) {
-    *q++ = MakeLeading2(ch >> 12);
-    *q++ = MakeContinuation(ch >> 6);
-    *q++ = MakeContinuation(ch);
-  } else if (ch < 0x200000) {
-    *q++ = MakeLeading3(ch >> 18);
-    *q++ = MakeContinuation(ch >> 12);
-    *q++ = MakeContinuation(ch >> 6);
-    *q++ = MakeContinuation(ch);
-  } else if (ch < 0x4000000) {
-    *q++ = MakeLeading4(ch >> 24);
-    *q++ = MakeContinuation(ch >> 18);
-    *q++ = MakeContinuation(ch >> 12);
-    *q++ = MakeContinuation(ch >> 6);
-    *q++ = MakeContinuation(ch);
-  } else if (ch < 0x80000000) {
-    *q++ = MakeLeading5(ch >> 30);
-    *q++ = MakeContinuation(ch >> 24);
-    *q++ = MakeContinuation(ch >> 18);
-    *q++ = MakeContinuation(ch >> 12);
-    *q++ = MakeContinuation(ch >> 6);
-    *q++ = MakeContinuation(ch);
-  } else {
-    // error
-  }
+	if (ch < 0x80) [[likely]] {
+		*q++ = (char)ch;
+	} else if (ch < 0x800) [[likely]] {
+		*q++ = MakeLeading1(ch >> 6);
+		*q++ = MakeContinuation(ch);
+	} else if (ch < 0x10000) {
+		*q++ = MakeLeading2(ch >> 12);
+		*q++ = MakeContinuation(ch >> 6);
+		*q++ = MakeContinuation(ch);
+	} else if (ch < 0x200000) {
+		*q++ = MakeLeading3(ch >> 18);
+		*q++ = MakeContinuation(ch >> 12);
+		*q++ = MakeContinuation(ch >> 6);
+		*q++ = MakeContinuation(ch);
+	} else if (ch < 0x4000000) {
+		*q++ = MakeLeading4(ch >> 24);
+		*q++ = MakeContinuation(ch >> 18);
+		*q++ = MakeContinuation(ch >> 12);
+		*q++ = MakeContinuation(ch >> 6);
+		*q++ = MakeContinuation(ch);
+	} else if (ch < 0x80000000) {
+		*q++ = MakeLeading5(ch >> 30);
+		*q++ = MakeContinuation(ch >> 24);
+		*q++ = MakeContinuation(ch >> 18);
+		*q++ = MakeContinuation(ch >> 12);
+		*q++ = MakeContinuation(ch >> 6);
+		*q++ = MakeContinuation(ch);
+	} else {
+		// error
+	}
 
-  return q;
+	return q;
 }
 
 std::size_t
