@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "TagSave.hxx"
@@ -34,10 +18,10 @@
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "io/BufferedOutputStream.hxx"
+#include "io/FileDescriptor.hxx"
 #include "io/StdioOutputStream.hxx"
-#include "util/ConstBuffer.hxx"
-#include "util/OptionDef.hxx"
-#include "util/OptionParser.hxx"
+#include "cmdline/OptionDef.hxx"
+#include "cmdline/OptionParser.hxx"
 #include "util/PrintException.hxx"
 
 #ifdef ENABLE_ARCHIVE
@@ -125,7 +109,7 @@ ParseCommandLine(int argc, char **argv)
 	}
 
 	auto args = option_parser.GetRemaining();
-	if (args.size != 1)
+	if (args.size() != 1)
 		throw std::runtime_error("Usage: run_input [--verbose] [--config=FILE] [--scan] [--chunk-size=BYTES] URI");
 
 	c.uri = args.front();
@@ -137,15 +121,14 @@ class GlobalInit {
 	EventThread io_thread;
 
 #ifdef ENABLE_ARCHIVE
-	const ScopeArchivePluginsInit archive_plugins_init;
+	const ScopeArchivePluginsInit archive_plugins_init{config};
 #endif
 
-	const ScopeInputPluginsInit input_plugins_init;
+	const ScopeInputPluginsInit input_plugins_init{config, io_thread.GetEventLoop()};
 
 public:
 	explicit GlobalInit(Path config_path)
-		:config(AutoLoadConfigFile(config_path)),
-		 input_plugins_init(config, io_thread.GetEventLoop())
+		:config(AutoLoadConfigFile(config_path))
 	{
 		io_thread.Start();
 	}
@@ -166,7 +149,7 @@ dump_input_stream(InputStream &is, FileDescriptor out,
 {
 	out.SetBinaryMode();
 
-	std::unique_lock<Mutex> lock(is.mutex);
+	std::unique_lock lock{is.mutex};
 
 	if (seek > 0)
 		is.Seek(lock, seek);
@@ -187,13 +170,13 @@ dump_input_stream(InputStream &is, FileDescriptor out,
 			}
 		}
 
-		char buffer[MAX_CHUNK_SIZE];
+		std::byte buffer[MAX_CHUNK_SIZE];
 		assert(chunk_size <= sizeof(buffer));
-		size_t num_read = is.Read(lock, buffer, chunk_size);
+		size_t num_read = is.Read(lock, {buffer, chunk_size});
 		if (num_read == 0)
 			break;
 
-		out.FullWrite(buffer, num_read);
+		out.FullWrite({buffer, num_read});
 	}
 
 	is.Check();
@@ -212,7 +195,7 @@ class DumpRemoteTagHandler final : public RemoteTagHandler {
 
 public:
 	Tag Wait() {
-		std::unique_lock<Mutex> lock(mutex);
+		std::unique_lock lock{mutex};
 		cond.wait(lock, [this]{ return done; });
 
 		if (error)
@@ -223,14 +206,14 @@ public:
 
 	/* virtual methods from RemoteTagHandler */
 	void OnRemoteTag(Tag &&_tag) noexcept override {
-		const std::scoped_lock<Mutex> lock(mutex);
+		const std::scoped_lock lock{mutex};
 		tag = std::move(_tag);
 		done = true;
 		cond.notify_all();
 	}
 
 	void OnRemoteTagError(std::exception_ptr e) noexcept override {
-		const std::scoped_lock<Mutex> lock(mutex);
+		const std::scoped_lock lock{mutex};
 		error = std::move(e);
 		done = true;
 		cond.notify_all();
