@@ -1,35 +1,21 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will  useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "output/Interface.hxx"
 #include "output/Registry.hxx"
 #include "output/OutputPlugin.hxx"
 #include "ConfigGlue.hxx"
+#include "lib/fmt/AudioFormatFormatter.hxx"
+#include "lib/fmt/RuntimeError.hxx"
 #include "event/Thread.hxx"
 #include "fs/Path.hxx"
 #include "fs/NarrowPath.hxx"
 #include "pcm/AudioParser.hxx"
 #include "pcm/AudioFormat.hxx"
-#include "util/OptionDef.hxx"
-#include "util/OptionParser.hxx"
+#include "cmdline/OptionDef.hxx"
+#include "cmdline/OptionParser.hxx"
+#include "io/FileDescriptor.hxx"
 #include "util/StringBuffer.hxx"
-#include "util/RuntimeError.hxx"
 #include "util/ScopeExit.hxx"
 #include "util/StaticFifoBuffer.hxx"
 #include "util/PrintException.hxx"
@@ -41,7 +27,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <stdio.h>
 
 struct CommandLine {
 	FromNarrowPath config_path;
@@ -76,13 +61,13 @@ ParseCommandLine(int argc, char **argv)
 	}
 
 	auto args = option_parser.GetRemaining();
-	if (args.size < 2 || args.size > 3)
+	if (args.size() < 2 || args.size() > 3)
 		throw std::runtime_error("Usage: run_output CONFIG NAME [FORMAT] <IN");
 
 	c.config_path = args[0];
 	c.output_name = args[1];
 
-	if (args.size > 2)
+	if (args.size() > 2)
 		c.audio_format = ParseAudioFormat(args[2], false);
 
 	return c;
@@ -95,19 +80,17 @@ LoadAudioOutput(const ConfigData &config, EventLoop &event_loop,
 	const auto *block = config.FindBlock(ConfigBlockOption::AUDIO_OUTPUT,
 					     "name", name);
 	if (block == nullptr)
-		throw FormatRuntimeError("No such configured audio output: %s",
-					 name);
+		throw FmtRuntimeError("No such configured audio output: {}",
+				      name);
 
 	const char *plugin_name = block->GetBlockValue("type");
 	if (plugin_name == nullptr)
 		throw std::runtime_error("Missing \"type\" configuration");
 
-	const auto *plugin = AudioOutputPlugin_get(plugin_name);
+	const auto *plugin = GetAudioOutputPluginByName(plugin_name);
 	if (plugin == nullptr)
-		throw FormatRuntimeError("No such audio output plugin: %s",
-					 plugin_name);
-#include "util/OptionDef.hxx"
-#include "util/OptionParser.hxx"
+		throw FmtRuntimeError("No such audio output plugin: {}",
+				      plugin_name);
 
 	return std::unique_ptr<AudioOutput>(ao_plugin_init(event_loop, *plugin,
 							   *block));
@@ -127,8 +110,7 @@ RunOutput(AudioOutput &ao, AudioFormat audio_format,
 	ao.Open(audio_format);
 	AtScopeExit(&ao) { ao.Close(); };
 
-	fprintf(stderr, "audio_format=%s\n",
-		ToString(audio_format).c_str());
+	fmt::print(stderr, "audio_format={}\n", audio_format);
 
 	const size_t in_frame_size = audio_format.GetFrameSize();
 
@@ -141,7 +123,7 @@ RunOutput(AudioOutput &ao, AudioFormat audio_format,
 			const auto dest = buffer.Write();
 			assert(!dest.empty());
 
-			ssize_t nbytes = in_fd.Read(dest.data, dest.size);
+			ssize_t nbytes = in_fd.Read(dest);
 			if (nbytes <= 0)
 				break;
 
@@ -151,13 +133,13 @@ RunOutput(AudioOutput &ao, AudioFormat audio_format,
 		auto src = buffer.Read();
 		assert(!src.empty());
 
-		src.size -= src.size % in_frame_size;
+		src = src.first(src.size() - src.size() % in_frame_size);
 		if (src.empty())
 			continue;
 
-		size_t consumed = ao.Play(src.data, src.size);
+		size_t consumed = ao.Play(src);
 
-		assert(consumed <= src.size);
+		assert(consumed <= src.size());
 		assert(consumed % in_frame_size == 0);
 
 		buffer.Consume(consumed);
