@@ -1,27 +1,10 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "config.h"
 #include "pcm/Export.hxx"
 #include "pcm/Traits.hxx"
 #include "util/ByteOrder.hxx"
-#include "util/ConstBuffer.hxx"
 
 #include <gtest/gtest.h>
 
@@ -46,14 +29,14 @@ TEST(PcmTest, ExportShift8)
 	EXPECT_EQ(e.GetInputBlockSize(), 8u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 8u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint8_t expected_silence[8]{};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -95,14 +78,14 @@ TEST(PcmTest, ExportPack24)
 	EXPECT_EQ(e.GetInputBlockSize(), 8u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 6u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(expected_size, dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(expected_size, dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint8_t expected_silence[6]{};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -134,9 +117,9 @@ TEST(PcmTest, ExportReverseEndian)
 	EXPECT_EQ(e.GetInputBlockSize(), 2u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 2u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(src), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, src, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(src), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), src, dest.size()) == 0);
 
 	e.Open(SampleFormat::S16, 2, params);
 
@@ -145,9 +128,9 @@ TEST(PcmTest, ExportReverseEndian)
 	EXPECT_EQ(e.GetInputBlockSize(), 4u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 4u);
 
-	dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected2), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected2, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected2), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected2, dest.size()) == 0);
 
 	e.Open(SampleFormat::S32, 2, params);
 
@@ -156,14 +139,14 @@ TEST(PcmTest, ExportReverseEndian)
 	EXPECT_EQ(e.GetInputBlockSize(), 8u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 8u);
 
-	dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected4), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected4, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected4), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected4, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint8_t expected_silence[8]{};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -199,33 +182,33 @@ TEST(PcmTest, ExportDsdU16)
 	EXPECT_EQ(e.GetInputBlockSize(), 4u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 4u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	/* no output, 2/4 remains */
 	static constexpr uint8_t src2[] = { 0x11, 0x22 };
-	dest = e.Export({src2, sizeof(src2)});
+	dest = e.Export(std::as_bytes(std::span{src2}));
 	EXPECT_TRUE(dest.empty());
 
 	/* one full frame and 2/4 remains */
 	static constexpr uint8_t src3[] = { 0x33, 0x44, 0x55, 0x66 };
 	static constexpr uint16_t expected3[] = { 0x1133, 0x2244 };
-	dest = e.Export({src3, sizeof(src3)});
-	EXPECT_EQ(sizeof(expected3), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected3, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src3}));
+	EXPECT_EQ(sizeof(expected3), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected3, dest.size()) == 0);
 
 	/* two full frames and 2/4 remains again */
 	static constexpr uint8_t src4[] = { 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee };
 	static constexpr uint16_t expected4[] = { 0x5577, 0x6688, 0x99bb, 0xaacc };
-	dest = e.Export({src4, sizeof(src4)});
-	EXPECT_EQ(sizeof(expected4), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected4, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src4}));
+	EXPECT_EQ(sizeof(expected4), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected4, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint8_t expected_silence[]{0x69, 0x69, 0x69, 0x69};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -259,33 +242,33 @@ TEST(PcmTest, ExportDsdU32)
 	EXPECT_EQ(e.GetInputBlockSize(), 8u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 8u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	/* no output, 4/8 remains */
 	static constexpr uint8_t src2[] = { 0x11, 0x22, 0x33, 0x44 };
-	dest = e.Export({src2, sizeof(src2)});
+	dest = e.Export(std::as_bytes(std::span{src2}));
 	EXPECT_TRUE(dest.empty());
 
 	/* one full frame and 4/8 remains */
 	static constexpr uint8_t src3[] = { 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc };
 	static constexpr uint32_t expected3[] = { 0x11335577, 0x22446688 };
-	dest = e.Export({src3, sizeof(src3)});
-	EXPECT_EQ(sizeof(expected3), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected3, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src3}));
+	EXPECT_EQ(sizeof(expected3), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected3, dest.size()) == 0);
 
 	/* two full frames and 2/4 remains again */
 	static constexpr uint8_t src4[] = { 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08 };
 	static constexpr uint32_t expected4[] = { 0x99bb0103, 0xaacc0204 };
-	dest = e.Export({src4, sizeof(src4)});
-	EXPECT_EQ(sizeof(expected4), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected4, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src4}));
+	EXPECT_EQ(sizeof(expected4), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected4, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint8_t expected_silence[]{0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -317,43 +300,43 @@ TEST(PcmTest, ExportDop)
 	EXPECT_EQ(e.GetInputBlockSize(), 8u);
 	EXPECT_EQ(e.GetOutputBlockSize(), 16u);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	/* not enough data: 2/8 */
 	static constexpr uint8_t src2[] = { 0x12, 0x34 };
-	dest = e.Export({src2, sizeof(src2)});
-	ASSERT_EQ(dest.size, 0u);
+	dest = e.Export(std::as_bytes(std::span{src2}));
+	ASSERT_EQ(dest.size(), 0u);
 
 	/* not enough data: 6/8 */
 	static constexpr uint8_t src3[] = { 0x56, 0x78, 0x9a, 0xbc };
-	dest = e.Export({src3, sizeof(src3)});
-	ASSERT_EQ(dest.size, 0u);
+	dest = e.Export(std::as_bytes(std::span{src3}));
+	ASSERT_EQ(dest.size(), 0u);
 
 	/* just enough data: 8/8 */
 	static constexpr uint8_t src4[] = { 0xde, 0xf0 };
 	static constexpr uint32_t expected4[] = { 0xff051256, 0xff053478, 0xfffa9ade, 0xfffabcf0 };
-	dest = e.Export({src4, sizeof(src4)});
-	ASSERT_EQ(sizeof(expected4), dest.size);
-	ASSERT_TRUE(memcmp(dest.data, expected4, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src4}));
+	ASSERT_EQ(sizeof(expected4), dest.size());
+	ASSERT_TRUE(memcmp(dest.data(), expected4, dest.size()) == 0);
 
 	/* not enough data: 6/8 */
 	static constexpr uint8_t src5[] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66 };
-	dest = e.Export({src5, sizeof(src5)});
+	dest = e.Export(std::as_bytes(std::span{src5}));
 	ASSERT_TRUE(dest.empty());
 
 	/* two quads returned, not enough data for more: 2/8 */
 	static constexpr uint8_t src6[] = { 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x10, 0x20 };
 	static constexpr uint32_t expected6[] = { 0xff051133, 0xff052244, 0xfffa5577, 0xfffa6688, 0xff0599bb, 0xff05aacc, 0xfffaddff, 0xfffaee00 };
-	dest = e.Export({src6, sizeof(src6)});
-	ASSERT_EQ(sizeof(expected6), dest.size);
-	ASSERT_TRUE(memcmp(dest.data, expected6, dest.size) == 0);
+	dest = e.Export(std::as_bytes(std::span{src6}));
+	ASSERT_EQ(sizeof(expected6), dest.size());
+	ASSERT_TRUE(memcmp(dest.data(), expected6, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr uint32_t expected_silence[]{0xff056969, 0xff056969, 0xfffa6969, 0xfffa6969};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
@@ -384,9 +367,9 @@ TestAlsaChannelOrder51()
 	PcmExport e;
 	e.Open(F, 6, params);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 }
 
 template<SampleFormat F, class Traits=SampleTraits<F>>
@@ -414,14 +397,14 @@ TestAlsaChannelOrder71()
 	PcmExport e;
 	e.Open(F, 8, params);
 
-	auto dest = e.Export({src, sizeof(src)});
-	EXPECT_EQ(sizeof(expected), dest.size);
-	EXPECT_TRUE(memcmp(dest.data, expected, dest.size) == 0);
+	auto dest = e.Export(std::as_bytes(std::span{src}));
+	EXPECT_EQ(sizeof(expected), dest.size());
+	EXPECT_TRUE(memcmp(dest.data(), expected, dest.size()) == 0);
 
 	const auto silence = e.GetSilence();
 	constexpr value_type expected_silence[8]{};
-	EXPECT_EQ(silence.size, sizeof(expected_silence));
-	EXPECT_EQ(memcmp(silence.data, expected_silence,
+	EXPECT_EQ(silence.size(), sizeof(expected_silence));
+	EXPECT_EQ(memcmp(silence.data(), expected_silence,
 			 sizeof(expected_silence)), 0);
 }
 
