@@ -3,23 +3,13 @@
  */
 
 #include "util/UriQueryParser.hxx"
-#include "util/StringView.hxx"
 
 #include <gtest/gtest.h>
 
-static bool
-operator==(StringView a, StringView b)
-{
-	if (a.IsNull() || b.IsNull())
-		return a.IsNull() == b.IsNull();
-
-	return a.Equals(b);
-}
-
 TEST(UriQueryParser, UriFindRawQueryParameter)
 {
 	const char *q = "foo=1&bar=2&quoted=%20%00+%%&empty1&empty2=";
-	EXPECT_EQ(UriFindRawQueryParameter(q, "doesntexist"),
+	EXPECT_EQ(UriFindRawQueryParameter(q, "doesntexist").data(),
 		  (const char *)nullptr);
 	EXPECT_EQ(UriFindRawQueryParameter(q, "foo"),
 		  "1");
